package cz.czu.bcp.trosa.bachelorsPrototype.controller;

import cz.czu.bcp.trosa.bachelorsPrototype.controller.utils.HttpHeadersFactory;
import cz.czu.bcp.trosa.bachelorsPrototype.dto.CreateClientDto;
import cz.czu.bcp.trosa.bachelorsPrototype.dto.RegisterDto;
import cz.czu.bcp.trosa.bachelorsPrototype.dto.UpdateClientDto;
import cz.czu.bcp.trosa.bachelorsPrototype.model.Client;
import cz.czu.bcp.trosa.bachelorsPrototype.model.Contract;
import cz.czu.bcp.trosa.bachelorsPrototype.model.User;
import cz.czu.bcp.trosa.bachelorsPrototype.service.ClientService;
import cz.czu.bcp.trosa.bachelorsPrototype.service.UserService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/client")

public class ClientController {
    private static final Logger console = LogManager.getLogger(ClientController.class);
    private final ClientService clientService;

    @Autowired
    public ClientController(ClientService clientService) {
        this.clientService = clientService;
    }

    @PreAuthorize("isAuthenticated()")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE,value = "/{id}")
    @ResponseStatus(code = HttpStatus.OK)
    public Client getById(@PathVariable(value = "id")Long id) {
        return this.clientService.get(id);

    }
    @PreAuthorize("isAuthenticated()")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(code = HttpStatus.OK)
    public List<Client> getAll() {
        return this.clientService.getAll();

    }
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
    @PreAuthorize("isAuthenticated()")
    @ResponseStatus(code = HttpStatus.CREATED)
    public void create(@RequestBody CreateClientDto body){
        var client = body.getClient();
        this.clientService.persist(client);
    }
    @PreAuthorize("isAuthenticated()")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE,value = "/{id}/contracts")
    @ResponseStatus(code = HttpStatus.OK)
    public List<Contract> getContracts(@PathVariable(value = "id")Long id) {
        return this.clientService.getContracts(id);
    }
    @PutMapping(consumes = MediaType.APPLICATION_JSON_VALUE,value ="/{id}" )
    @PreAuthorize("isAuthenticated()")
    @ResponseStatus(code = HttpStatus.NO_CONTENT)
    public void update(@RequestBody UpdateClientDto body,@PathVariable(value = "id")Long id){
        this.clientService.update(body,id);
    }
    @PreAuthorize("isAuthenticated()")
    @DeleteMapping(value ="/{id}")
    @ResponseStatus(code = HttpStatus.NO_CONTENT)
    public void deleteClient(@PathVariable(value = "id")Long id){
        this.clientService.delete(id);
    }


}


