package cz.czu.bcp.trosa.bachelorsPrototype.service;

import cz.czu.bcp.trosa.bachelorsPrototype.dto.UpdateClientDto;
import cz.czu.bcp.trosa.bachelorsPrototype.model.Client;
import cz.czu.bcp.trosa.bachelorsPrototype.model.Contract;
import cz.czu.bcp.trosa.bachelorsPrototype.model.User;
import cz.czu.bcp.trosa.bachelorsPrototype.repository.ClientRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityExistsException;
import javax.persistence.EntityNotFoundException;
import javax.persistence.Id;
import javax.transaction.Transactional;
import java.util.List;

@Service
public class ClientService {
    private final ClientRepository clientRep;

    @Autowired
    public ClientService(ClientRepository clientRep) {
        this.clientRep = clientRep;
    }

    public List<Client> getAll() {
        return (List<Client>) clientRep.findAll();
    }

    public Client get(Long id) {
        var temp = clientRep.findById(id);
        if (temp.isEmpty()) {
            throw new EntityNotFoundException();
        }
        return temp.get();
    }
    @Transactional
    public void persist(Client c) throws EntityExistsException {
        if(clientRep.getByEmail(c.getEmail()).isPresent()){
            throw new EntityExistsException();
        }
        clientRep.save(c);
    }
    public List<Contract> getContracts(Long id){
        var client = get(id);
        return  client.getContracts();
    }
    @Transactional
    public void update(UpdateClientDto c, Long id) {
        var client = get(id);
        if (clientRep.getByEmail(c.email).isPresent()) {
            throw new EntityExistsException();
        }
        if (c.city!=null){
            client.setCity(c.city);
        }
        if (c.houseNumber!=null){
            client.setHouseNumber(c.houseNumber);
        }
        if (c.street!=null){
            client.setStreet(c.street);
        }
        if (c.countryCode!=null){
            client.setCountryCode(c.countryCode);
        }
        if (c.zipCode!=0){
            client.setZipCode(c.zipCode);
        }
        if (c.surname!=null){
            client.setSurname(c.surname);
        }
        if (c.firstname!=null){
            client.setFirstname(c.firstname);
        }
        if (c.personalNumber!=0){
            client.setPersonalNumber(c.personalNumber);
        }
        if (c.email!=null){
            client.setEmail(c.email);
        }
        if (c.phoneNumber!=0){
            client.setPhoneNumber(c.phoneNumber);
        }
        clientRep.save(client);
    }
    @Transactional
    public void delete(Long id){
        var client = get(id);
        clientRep.delete(client);
    }
}

