package cz.czu.bcp.trosa.bachelorsPrototype.service;

import cz.czu.bcp.trosa.bachelorsPrototype.dto.CreateContractDto;
import cz.czu.bcp.trosa.bachelorsPrototype.dto.UpdateContractDto;
import cz.czu.bcp.trosa.bachelorsPrototype.dto.UpdateUserDto;
import cz.czu.bcp.trosa.bachelorsPrototype.model.Contract;
import cz.czu.bcp.trosa.bachelorsPrototype.repository.ClientRepository;
import cz.czu.bcp.trosa.bachelorsPrototype.repository.ContractRepository;
import cz.czu.bcp.trosa.bachelorsPrototype.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Service;

import javax.persistence.EntityExistsException;
import javax.persistence.EntityNotFoundException;
import javax.transaction.Transactional;
import java.util.List;

@Service
public class ContractService {
    private final ContractRepository contractRep;
    private final UserRepository userRep;
    private final ClientRepository clientRep;

    @Autowired
    public ContractService(ContractRepository contractRep, UserRepository userRep, ClientRepository clientRep) {
        this.contractRep = contractRep;
        this.userRep = userRep;
        this.clientRep = clientRep;
    }

    public List<Contract> getAll() {
        return (List<Contract>) contractRep.findAll();
    }

    public Contract get(Long id) {
        var temp = contractRep.findById(id);
        if (temp.isEmpty()) {
            throw new EntityNotFoundException();
        }
        return temp.get();
    }
    @Transactional
    public void persist(CreateContractDto c, Authentication auth) throws EntityExistsException {
        assert (c.dueDate.isAfter(c.startDate));
        var contract= c.getContract();
        var user = userRep.getByUsername(auth.getName());
        var client = clientRep.findById(c.clientId);
        if (user.isEmpty()||client.isEmpty())
            throw new EntityNotFoundException();
        contract.setClient(client.get());
        contract.setProvider(user.get());
        contractRep.save(contract);
    }
    @Transactional
    public void update(UpdateContractDto c,Long id){
        var contract = get(id);
        if (c.newlyPaidAmount!=null){
            contract.setRemainingAmount(contract.getRemainingAmount().subtract(c.newlyPaidAmount));
        }
        if (c.dueDate!=null){
            contract.setDueDate(c.dueDate);
        }
        if (c.overdueAmount!=null){
            contract.setOverdueAmount(c.overdueAmount);
        }
        contractRep.save(contract);
    }
}




