package cz.czu.bcp.trosa.bachelorsPrototype.service;

import cz.czu.bcp.trosa.bachelorsPrototype.model.User;
import cz.czu.bcp.trosa.bachelorsPrototype.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.security.core.userdetails.UserDetailsService;

import javax.persistence.EntityExistsException;
import javax.transaction.Transactional;
import java.util.ArrayList;

@Service
public class UserService implements UserDetailsService {
    private final UserRepository userRep;
    private final BCryptPasswordEncoder bcrypt;

    @Autowired
    public UserService(UserRepository userRep, BCryptPasswordEncoder bcrypt) {
        this.userRep = userRep;
        this.bcrypt = bcrypt;
    }

    @Override
    public UserDetails loadUserByUsername(String s) throws UsernameNotFoundException {
        var details=userRep.getByUsername(s);
        if (details.isEmpty()){
            throw new UsernameNotFoundException(s);
        }
        var temp = new ArrayList<SimpleGrantedAuthority>();
        temp.add(new SimpleGrantedAuthority("User"));
        return new org.springframework.security.core.userdetails.User(details.get().getUsername(),details.get().getPassword(),temp);
    }

    @Transactional
    public void persist(User u) throws EntityExistsException{
        if (userRep.getByUsername(u.getUsername()).isPresent() || userRep.getByEmail(u.getEmail()).isPresent()) {
            throw new EntityExistsException();
        }
        u.setPassword(bcrypt.encode(u.getPassword()));
        userRep.save(u);

    }
    public User findByUsername(String username){
        return userRep.getByUsername(username).get();
    }
    @Transactional
    public void createAdmin(){
        var user = new User();
        user.setUsername("Admin");
        user.setEmail("example@example.com");
        user.setPassword(bcrypt.encode("Password"));
        userRep.save(user);
    }

}

